package rpc

const RPCPath = "/ecksdee/api"
const RPCName = "XD"

const RPCListTorrents = RPCName + ".ListTorrents"
const RPCListTorrentStatus = RPCName + ".SwarmStatus"
const RPCTorrentStatus = RPCName + ".TorrentStatus"
const RPCAddTorrent = RPCName + ".AddTorrent"
const RPCDelTorrent = RPCName + ".DelTorrent"
const RPCSetPieceWindow = RPCName + ".SetPieceWindow"
const RPCChangeTorrent = RPCName + ".ChangeTorrent"
const RPCSwarmCount = RPCName + ".SwarmCount"
