package i2p

// default address of i2p interface
const DEFAULT_ADDRESS = "127.0.0.1:7656"

// default path to private keys
const DEFAULT_KEYFILE = "xd-privkey.dat"

// default i2p session name
const DEFAULT_NAME = "XD"
