package extensions

// I2PPeerExchange is a BitTorrent Extension indicating we support PEX variant for i2p
const I2PPeerExchange = Extension("i2p_pex")

// LokinetPeerExchange is a Bittorrent Extension indication we support Lokinet PEX
const LokinetPeerExchange = Extension("ln_pex")
