body {
    background: #2E3436;
    color: #D3D7CF;
}

.base.container {
    max-width: none;
    padding: 24px;
}

nav {
    padding-bottom: 20px;
}

nav > div.row {
    margin: 20px 0px;
}

.speed {
    color: #D3D7CF;
}

.speed:hover {
    color: #ccc;
    background-color: #222;
    cursor: pointer;
}

progress {
    width: 100%;
}

.logo img {
    width: 38px;
}

.button-remove {
    color: #D3D7CF;
    font-weight: bold;
    padding: 0px 10px;
    border-radius: 4px;
    border:1px #D3D7CF solid;
}

.button-remove:hover {
    cursor: pointer;
    background: #D3D7CF;
    color: #ccc;
}

.torrent-row {
    padding: 10px;
    font-size: 0.8em;
    border: 1px solid #222;
    border-top: none;
}

.torrent-row:first-child {
    border-top: 1px solid #222;
}

.torrent-button {
    text-align: right;
    padding-bottom: 1em;
}

.torrent-row:nth-child(odd) {
    background: #272C2D;
}

.torrent-row:hover {
    background: #222;
}

.torrent-row-downloading {
    color: #D3D7CF;
}

.torrent-row-seeding {
    color: #00BB00;
}

progress {
    -webkit-appearance: none;
     appearance: none;
}

.filter-buttons li {
    display: inline-block;
    font-size: 0.7em;
    height: 38px;
    padding: 0 10px;
    color: #D3D7CF;
    text-align: center;
    line-height: 38px;
    letter-spacing: .1rem;
    text-transform: uppercase;
    text-decoration: none;
    white-space: nowrap;
    background-color: #272C2D;
    border-radius: 4px;
    border: 1px solid #D3D7CF;
    cursor: pointer;
    margin: 0px 2px;
    box-sizing: border-box;
}

.filter-buttons li.active {
    background: #D3D7CF;
    color: #272C2D;
}

.filter-buttons li:hover {
    background: #D3D7CF;
    color: #272C2D;
}

.button.button-primary,
button.button-primary,
input[type="submit"].button-primary,
input[type="reset"].button-primary,
input[type="button"].button-primary {
  color: #272C2D !important;
  background-color: #D3D7CF !important;
  border-color: #D3D7CF !important; }
.button.button-primary:hover,
button.button-primary:hover,
input[type="submit"].button-primary:hover,
input[type="reset"].button-primary:hover,
input[type="button"].button-primary:hover,
.button.button-primary:focus,
button.button-primary:focus,
input[type="submit"].button-primary:focus,
input[type="reset"].button-primary:focus,
input[type="button"].button-primary:focus {
  color: #FFF !important;
  background-color: #00BB00 !important;
  border-color: #00BB00 !important; }

.button.button-cancel:hover,
button.button-cancel:hover,
input[type="submit"].button-cancel:hover,
input[type="reset"].button-cancel:hover,
input[type="button"].button-cancel:hover,
.button.button-cancel:focus,
button.button-cancel:focus,
input[type="submit"].button-cancel:focus,
input[type="reset"].button-cancel:focus,
input[type="button"].button-cancel:focus {
  color: #FFF !important;
  background-color: #CB2431 !important;
  border-color: #CB2431 !important; }

input[type="email"],
input[type="number"],
input[type="search"],
input[type="text"],
input[type="tel"],
input[type="url"],
input[type="password"],
textarea,
select {
  background-color: #272C2D !important;
  border: 1px solid #D3D7CF !important;
  color: #D3D7CF !important; }
input[type="email"]:focus,
input[type="number"]:focus,
input[type="search"]:focus,
input[type="text"]:focus,
input[type="tel"]:focus,
input[type="url"]:focus,
input[type="password"]:focus,
textarea:focus,
select:focus {
  border: 1px solid #00BB00 !important;
  color: #00BB00 !important; }

/* The Modal (background) */
.confirmation-box {
    position: fixed;
    z-index: 1;
    left: 0;
    top: 0;
    width: 100%;
    height: 100%;
    overflow: auto;
    background-color: rgb(0,0,0);
    background-color: rgba(0,0,0,0.4);
}

/* Modal Content/Box */
.confirmation-content {
    background-color: #272C2D;
    color: #D3D7CF;
    margin: 15% auto;
    padding: 20px;
    border: 1px solid #888;
    width: 80%;
}
